/*
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
| File Name			  : MicroBolt_I2C_GPIO.c																					
| Author			    : Micromint, Inc.						
| Copyright       : Copyright  2005, Micromint, Inc.																	
| Creation Date		: 12/30/05																							
| Version			    : 1.00																																												
| Spaces per tab	:	2																									
| Description		  : Main C file																																									 
| Revision			  :	Initial																																																						
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
*/

/*
|----------------------------------------------------
|     Includes
|----------------------------------------------------
*/

#include <ARM/philips/lpc210x.h>
#include <arm_macros.h>
#include "MicroBolt_I2C_GPIO.h"

static unsigned char I2cSlaveAddress = 0;       // I2C slave address
static unsigned char I2cBuffer[20];             // I2C application buffer
static unsigned char I2cBufferIndex = 0;        // Index for I2C buffer
static unsigned char I2cPacketDataSize = 0;     // Number of data bytes for an I2C packet
static unsigned char I2cPacketType = 0;         // Read or Write packet type
static unsigned char I2cPacketInProgress = 0;	  // I2C Packet in progress flag			

/*
|------------------------------------------------------------------------------------------
|     Function        :   main
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Main function for system
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

void main(void)
  {
  unsigned char PCA9557portValue = 0;

  __DISABLE_INTERRUPT();                        // Disable all interrupts

/*
|-------------------------
| Config MAM 
|-------------------------
*/
  MAM_CR = 0x00;                                // Turn MAM off (default)
  MAM_TIM = 0x04;                               // Set flash timing to 4 clock cycles

  MAM_CR = 0x02;                                // Fully enable the Memory Accleration Module

/*
|-------------------------
| Config PLL and CCLK 
|-------------------------
*/
  SCB_PLLCFG |= 0x23;				 	   		            // Set to 59 MHz (0x03 is multiply value of 4)
  SCB_PLLCON |= 0x01;                           // Enable the PLL 
  SCB_PLLFEED = 0xAA;					   	  	          // Shadow register copy to enable changes
  SCB_PLLFEED = 0x55;                           // in PLLCON and PLLCFG

/*
|-------------------------
| Config PCLK 
|-------------------------
*/
  SCB_VPBDIV = 0;                               // Peripheral clock is 1/4th Processor clock which equals 14.7456 MHz

/*
|-------------------------
| Configure VIC 
|-------------------------
*/
  VICVectAddr0 = (unsigned)pll_isr;             // Assign the PLL lock ISR vector address
  VICVectCntl0 = INTERRUPT_CHANNEL_FOR_PLL;     // Assign the VIC address to the actual interrupt
  VICIntEnable = INTERRUPT_ENABLE_FOR_PLL;      // Enable the interrupt  

	VICVectAddr1 = (unsigned)I2c_ISR;							// Assign the I2C ISR vector address 
  VICVectCntl1 = INTERRUPT_CHANNEL_FOR_I2C;     // Assign the VIC address to the actual interrupt
  VICIntEnable = INTERRUPT_ENABLE_FOR_I2C;      // Enable the interrupt 

  __ENABLE_INTERRUPT();                         // Enable all interrupts

/*
|-------------------------
| Config GPIO 
|-------------------------
*/
  PCB_PINSEL0=0x00000050;                       // Setup with ICCARM App builder - MicroBolt_I2C_GPIO.bcf (I2C)   
  PCB_PINSEL1=0x55400000;                       //                                   (Secondary JTAG pins) 

  GPIO_IODIR |= MICROBOLT_LED;                  // Setup MicroBolt LED as output

  GPIO_IOCLR=0xffffffff;                        // Clear all pins to start with

/*
|-------------------------
| Config I2C Master
|-------------------------
*/
  I2C_I2SCLH = 37;                              // I2C clock is 200 KHz (14.7456 MHz/(SCLH + SCLL))
  I2C_I2SCLL = 37;

  I2C_I2CONSET = I2C_ENABLE_BIT;                // Enable the I2C channel

/*
|-----------------------------------------
| Config I2C PCA9557 Slave device I/O port
|-----------------------------------------
*/                                              // Initialize I2C buffers with desired data for PCA9557 slave
                                                // The below configuration sets up the PCA9557 port pins   
  I2cSlaveAddress = I2C_SLAVE_ADDR_PCA9557;     // Address the PCA9557
  I2cPacketDataSize = 2;                        // How many bytes from the I2c buffer to send
  I2cBuffer[0] = 0x03;                          // Control register value 0x03 selects the configuration register to define I/O port
  I2cBuffer[1] = 0xF3;                          // IO-2 and IO-3 are outputs and the rest are inputs (0's outputs, 1's inputs) 
  I2cStart(I2C_WRITE);                          // Send out an I2C Start condition for a Write packet
  while(I2cPacketInProgress == TRUE);           // Wait here for I2C packet to complete 

/*
|--------------------------------------------------
| Config I2C PCA9557 Slave device polarity register
|--------------------------------------------------
*/                                              // Initialize I2C buffers with desired data for PCA9557 slave
                                                // The below configuration sets up the PCA9557 polarity inversion register  
  I2cSlaveAddress = I2C_SLAVE_ADDR_PCA9557;     // Address the PCA9557
  I2cPacketDataSize = 2;                        // How many bytes from the I2c buffer to send
  I2cBuffer[0] = 0x02;                          // Control register value 0x02 selects the polarity inversion register 
  I2cBuffer[1] = 0x00;                          // All polarities match actual logic levels 
  I2cStart(I2C_WRITE);                          // Send out an I2C Start condition for a Write packet
  while(I2cPacketInProgress == TRUE);           // Wait here for I2C packet to complete 

/*
|----------------------------------------------------
|     Start of application
|----------------------------------------------------
*/
  while(1)                                      // Do this forever
    {

/*
|------------------------------------------------
| Read PCA9557 I2C Slave device's input register
|------------------------------------------------
*/  
    PCA9557portValue = ReadPCA9557IoPort();     // Read 8-bit port value from the PCA9557 via I2C

/*
|-----------------------------------------------------------
| Write to the PCA9557 port based upon the value read above
|-----------------------------------------------------------
*/ 
	  switch(PCA9557portValue & 0xF0)             // Take port value read from PCA9557 and mask off lower nibble
			  {                                       // High nibble IO-4 to IO-7 is now what's left in PCA9557portValue
			  case (0x10):	                          // Is I0-4 the only one high?
            {
            WritePCA9557IoPort(0x04);           // Yes, Turn PCA9557 IO-2 LED On
            GPIO_IOCLR = MICROBOLT_LED;	        // Keep the MicroBolt LED Off
            break;
            }
			  case (0x20):	                          // Is I0-5 the only one high?
            {
            WritePCA9557IoPort(0x08);           // Yes, Turn PCA9557 IO-3 LED On
            GPIO_IOCLR = MICROBOLT_LED;	        // Keep the MicroBolt LED Off
            break;
            }
			  case (0x40):	                          // Is I0-6 the only one high?
            {
            WritePCA9557IoPort(0x0C);           // Yes, Turn PCA9557 IO-2 & IO-3 LED's On
            GPIO_IOCLR = MICROBOLT_LED;	        // Keep the MicroBolt LED Off
            break;
            }
			  case (0x80):	                          // Is I0-7 the only one high?
            {
            WritePCA9557IoPort(0x00);           // Yes, Turn PCA9557 IO-2 & IO-3 LED's Off
            GPIO_IOSET = MICROBOLT_LED;	        // Turn the MicroBolt LED On
            break;
            }
			  case (0xF0):	                          // Are I0-4, IO-5, IO-6, and IO-7 all high?
            {
            WritePCA9557IoPort(0x0C);           // Yes, Turn PCA9557 IO-2 & IO-3 LED's On
            GPIO_IOSET = MICROBOLT_LED;	        // Turn the MicroBolt LED On
            break;
            }
			  default:                                // Any other combination of IO-4 through IO-7 ends up here
					  {
            WritePCA9557IoPort(0x00);           // Turn PCA9557 IO-2 & IO-3 LED's Off
            GPIO_IOCLR = MICROBOLT_LED;         // Keep the MicroBolt LED Off
					  break;
					  }
        }
    }
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   WritePCA9557IoPort
|     Inputs          :   IoPortPinValue
|     Outputs         :   None
|     Purpose         :   Write an 8 bit value to the PCA9557 IO port
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
void WritePCA9557IoPort(unsigned char IoPortPinValue)
  {
  I2cSlaveAddress = I2C_SLAVE_ADDR_PCA9557;     // Address the PCA9557
  I2cPacketDataSize = 2;                        // How many bytes from the I2c buffer to send
  I2cBuffer[0] = 0x01;                          // Control register value 0x01 selects the output register 
  I2cBuffer[1] = IoPortPinValue;                // Desired IO port value 
  I2cStart(I2C_WRITE);                          // Send out an I2C Start condition for a Write packet
  while(I2cPacketInProgress == TRUE);           // Wait here for I2C packet to complete
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   ReadPCA9557IoPort
|     Inputs          :   None
|     Outputs         :   PCA9557 8-bit port value
|     Purpose         :   Read an 8 bit value from the PCA9557 IO port
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
unsigned char ReadPCA9557IoPort(void)
  {
  I2cSlaveAddress = I2C_SLAVE_ADDR_PCA9557;     // Address the PCA9557
  I2cPacketDataSize = 1;                        // How many bytes from the I2c buffer to send
  I2cBuffer[0] = 0x00;                          // Control register value 0x00 selects input register for reading port pins 
  I2cStart(I2C_WRITE);                          // Send out an I2C Start condition for a Write packet
  while(I2cPacketInProgress == TRUE);           // Wait here for previous I2C packet to complete 

  I2cSlaveAddress = I2C_SLAVE_ADDR_PCA9557;     // Address the PCA9557
  I2cPacketDataSize = 1;                        // How many bytes from the I2c buffer to receive
  I2cStart(I2C_READ);                           // Send out an I2C Start condition for a Read packet 
  while(I2cPacketInProgress == TRUE);           // Wait here for previous I2C packet to complete 

  return(I2cBuffer[0]);                         // Return the byte read from the PCA9557 IO port
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   I2c_ISR
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   I2C interrupt and command processing
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
#pragma interrupt_handler I2c_ISR

void I2c_ISR(void)
{
	switch(I2C_I2STAT)
			{
//****************************************************
// I2C Master Transmit Service Routines
//****************************************************
			case (0x08):																									// START condition transmitted
					{
					I2cBufferIndex = 0;																			  // Reset index counter
          I2C_I2DAT = I2cSlaveAddress | I2cPacketType;              // Desired slave address into I2C register OR'd with Read or Write
      	  I2C_I2CONCLR = I2C_STOP_BIT;	                            // Clear stop bit per datasheet
					break;
					}
//------------------------------------
			case (0x10):																									// Repeated START condition transmitted
					{
					I2cBufferIndex = 0;																			  // Reset index counter
          I2C_I2DAT = I2cSlaveAddress | I2cPacketType;              // Desired slave address into I2C register OR'd with Read or Write
      	  I2C_I2CONCLR = I2C_STOP_BIT;	                            // Clear stop bit per datasheet
					break;
					}
//------------------------------------
			case (0x18):																									// Slave address with WRITE transmitted and ACK received
					{
          I2C_I2DAT = I2cBuffer[I2cBufferIndex];                    // Put data from I2c buffer into I2C register for sending 
          I2cBufferIndex++;                                         // Increment the buffer index
      	  I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		          // Clear start and stop bits per datasheet        
  				break;
					}
//------------------------------------
			case (0x28):																									// Data byte transmitted and ACK received
					{
          I2C_I2DAT = I2cBuffer[I2cBufferIndex];                    // Put data from I2c buffer into I2C register for sending  
          I2cBufferIndex++;                                         // Increment the buffer index
      	  I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		          // Clear start and stop bits per datasheet  

          if (I2cBufferIndex > I2cPacketDataSize)                   // All the bytes from the I2C buffer sent out?
      	      {
              I2C_I2CONCLR = I2C_START_BIT;                         // Yes, Clear start bit per datasheet  
              I2cStop();                                            // Done sending data, send a stop condition to end packet
              }
					break;
					}
//------------------------------------
			case (0x20):																									// Slave address transmitted and NAK received
					{
          I2C_I2CONCLR = I2C_START_BIT;                             // Clear start bit per datasheet  
          I2cStop();                                                // Done sending data, send a stop condition to end packet
					break;
					}
//------------------------------------
			case (0x30):																									// Data byte transmitted and NAK received
					{
          I2C_I2CONCLR = I2C_START_BIT;                             // Clear start bit per datasheet  
          I2cStop();                                                // Done sending data, send a stop condition to end packet
					break;
					}
//------------------------------------
			case (0x38):																									// Master lost arbitration 
					{                                                         // Could send another START if desirable
      	  I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		          // Clear start and stop bits per datasheet 
					break;
					}
//------------------------------------

//****************************************************
// I2C Master Receive Service Routines
//****************************************************
			case (0x40):																									// Slave address with READ transmitted and ACK received
					{
					if (I2cPacketDataSize <= 1)		                            // Only one byte for the packet?
					    {
	            I2C_I2CONCLR = I2C_AA_BIT;                            // Send a NAK, don't want any more bytes
							I2cBufferIndex = 0;													          // Reset count index
							}
					else
							{
	            I2C_I2CONSET |= I2C_AA_BIT;                           // Send an ACK
						  }

      	  I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		          // Clear start and stop bits per datasheet            
					break;
					}
//------------------------------------
			case (0x50):																									// Data byte received and ACK returned
					{                                                         
					I2cBuffer[I2cBufferIndex] = I2C_I2DAT;									  // Grab the I2C data and put it into the I2C buffer
          I2cBufferIndex++;																          // Increment buffer count index
					if (I2cBufferIndex < (I2cPacketDataSize - 1))             // Still more data to come for packet?
					    {
	            I2C_I2CONSET |= I2C_AA_BIT;                           // Yes, Send an ACK
							}
					else
							{
	            I2C_I2CONCLR = I2C_AA_BIT;                            // No, Send a NAK, don't want any more bytes
							I2cBufferIndex = 0;													          // Reset count index
						  }

      	  I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		          // Clear start and stop bits per datasheet 
					break;
					}
//------------------------------------
			case (0x48):																									// Slave address with READ transmitted and NAK received
					{
          I2C_I2CONCLR = I2C_START_BIT;                             // Clear start bit per datasheet  
          I2cStop();                                                // Send a stop condition to end packet
					break;
					}
//------------------------------------
			case (0x58):																									// Data byte received and NAK returned
					{           
					I2cBuffer[I2cBufferIndex] = I2C_I2DAT;									  // Grab the I2C data and put it into the I2C buffer                                              
          I2C_I2CONCLR = I2C_START_BIT;                             // Clear start bit per datasheet  
          I2cStop();                                                // Send a stop condition to end packet
					break;
					}
//------------------------------------
			default:
					{
      	  I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		          // Clear start and stop bits per datasheet 
					break;
					}
			}
//------------------------------------

	I2C_I2CONCLR = I2C_SI_BIT;																		    // Clear I2C SI interrupt flag 
	VICVectAddr = VIC_ACK; 																						// Acknowledge Interrupt
}

/*
|------------------------------------------------------------------------------------------
|     Function        :   I2cStart
|     Inputs          :   I2cReadOrWriteNot
|     Outputs         :   None
|     Purpose         :   Send a start condition and setup packet type (Read or Write)
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
void I2cStart(unsigned char I2cReadOrWriteNot)
  {
  I2cPacketInProgress = TRUE;                   // Flag for stating an I2C packet is about to happen
  I2cPacketType = I2cReadOrWriteNot;            // Flag for stating what kind of packet it is; Read or Write
	I2C_I2CONSET |= I2C_START_BIT;                // Set the start bit 
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   I2cStop
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Send a stop condition
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
void I2cStop(void)
  {
	I2C_I2CONSET |= I2C_STOP_BIT;                 // Set the stop bit 
  I2cPacketInProgress = FALSE;                  // Flag for stating I2C packet is done
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   pll_isr
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Once PLL has locked, connect it and use for system clock
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
#pragma interrupt_handler pll_isr

void pll_isr(void)
  {
  SCB_PLLCON |= 0x02;                           // Connect the PLL 
  SCB_PLLFEED = 0xAA;					   	  	          // Shadow register copy to enable changes
  SCB_PLLFEED = 0x55;                           // in PLLCON and PLLCFG
  VICIntEnClear = PLL_CLR;                      // Clear PLL interrupt flag
  VICVectAddr = VIC_ACK;                        // Acknowledge Interrupt
  }